MalygosStacks = LibStub("AceAddon-3.0"):NewAddon("MalygosStacks", "AceEvent-3.0", "AceTimer-3.0")

local vehicleId
local comboPoints
local spentPoints
local debuffStacks
local expirationTime


function MalygosStacks:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("MalygosStacksDB")
end

function MalygosStacks:OnEnable()
	self:RegisterEvent("UNIT_ENTERED_VEHICLE")
	self:RegisterEvent("UNIT_EXITED_VEHICLE")
	
	vehicleId = nil
end

function MalygosStacks:OnDisable()
	if BWMalygosStacks then
		BWMalygosStacks:Hide()
	end
	
	vehicleId = nil
end

function MalygosStacks:SetupFrames()
	BWMalygosStacks = CreateFrame("Frame", "BWMalygosStacksFrame", UIParent)
	BWMalygosStacks:SetWidth(64)
	BWMalygosStacks:SetHeight(64)
	BWMalygosStacks:ClearAllPoints()
	BWMalygosStacks:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	
	local texture = BWMalygosStacks:CreateTexture("BWMalygosStacksBackground", "BACKGROUND")
	texture:SetBlendMode("BLEND")
	texture:SetAllPoints(BWMalygosStacks)
	texture:SetTexture("Interface\\Icons\\Spell_Fire_Burnout")
	
	BWMalygosStacks:EnableMouse(true)
	BWMalygosStacks:SetMovable(true)
	BWMalygosStacks:SetClampedToScreen(true)
	BWMalygosStacks:RegisterForDrag("LeftButton")
	BWMalygosStacks:SetScript("OnDragStart", function()
		this:StartMoving()
	end)
	BWMalygosStacks:SetScript("OnDragStop", function()
		this:StopMovingOrSizing()
		self:SavePosition()
	end)
	
	local text1 = BWMalygosStacks:CreateFontString(nil, "OVERLAY")
	text1:ClearAllPoints()
	text1:SetPoint("TOP", BWMalygosStacks, "BOTTOM", 0, 0)
	text1:SetJustifyH("CENTER")
	text1:SetFont("Fonts\\FRIZQT__.TTF", 12, "OUTLINE")
	BWMalygosStacks.time = text1
	
	local text2 = BWMalygosStacks:CreateFontString(nil, "OVERLAY")
	text2:SetWidth(64)
	text2:SetHeight(64)
	text2:ClearAllPoints()
	text2:SetPoint("TOPLEFT", BWMalygosStacks, "TOPLEFT", 0, 0)
	text2:SetJustifyH("CENTER")
	text2:SetJustifyV("MIDDLE")
	text2:SetFont("Fonts\\FRIZQT__.TTF", 36, "OUTLINE")
	text2:SetText("|cFFFFFFFF0")
	BWMalygosStacks.count = text2
	
	BWMalygosStacks:Hide()
	
	--ripped from PitBull_ComboPoints by ckknight
	local comboPoints = CreateFrame("Frame", "BWMalygosComboPoints", BWMalygosStacks)
	comboPoints:SetWidth(60)
	comboPoints:SetHeight(12)
	comboPoints:SetPoint("BOTTOMRIGHT", BWMalygosStacks, "BOTTOMRIGHT", -2, 0)
	for i = 1, 5 do
		local combo = comboPoints:CreateTexture(nil, "OVERLAY")
		combo:SetTexture("Interface\\AddOns\\MalygosStacks\\combo")
		combo:SetWidth(12)
		combo:SetHeight(12)
		combo:ClearAllPoints()
		if i == 1 then
			combo:SetPoint("RIGHT", comboPoints, "RIGHT")
		else
			combo:SetPoint("RIGHT", comboPoints["combo"..(i-1)], "LEFT")
		end
		combo:Hide()
		comboPoints["combo"..i] = combo
	end
	BWMalygosStacks.comboPoints = comboPoints
	
	local x = self.db.char.posx
	local y = self.db.char.posy
	if x and y then
		local s = BWMalygosStacks:GetEffectiveScale()
		BWMalygosStacks:ClearAllPoints()
		BWMalygosStacks:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x / s, y / s)
	end
end

function MalygosStacks:SavePosition()
	local s = BWMalygosStacks:GetEffectiveScale()
	self.db.char.posx = BWMalygosStacks:GetLeft() * s
	self.db.char.posy = BWMalygosStacks:GetTop() * s
end

function MalygosStacks:Update()
	local power = UnitPower("vehicle")
	if power >= 50 then
		BWMalygosStacksBackground:SetVertexColor(1, 1, 1, 1)
	else
		--ramp up to full alpha at 50 energy, but provide a visual jump (.8 > 1) at 50
		BWMalygosStacksBackground:SetVertexColor(1, 1, 1, max(0.4, (power-10)*2/100))
	end
	
	local remaining = expirationTime - GetTime()	
	if debuffStacks and remaining >= 0 then
		local color = "00FF00"
		if remaining <= 3 then
			color = "FF0000" --red
		elseif remaining <= 6 then
			color = "FFAA00" --dark orange
		elseif remaining <= 7.5 then
			color = "FFD500" --light orange
		elseif remaining <= 9 then
			color = "FFFF00" --yellow
		elseif remaining <= 10.5 then
			color = "D5FF00" --lighter green
		elseif remaining <= 12 then
			color = "AAFF00"--lighter green
		elseif remaining <= 13.5 then
			color = "55FF00" --lighter green
		elseif remaining <= 15 then
			color = "2BFF00" --almost green
		end
		
		BWMalygosStacks.time:SetFormattedText("|cFF%s%0.1f s|r", color, remaining)
		BWMalygosStacks.time:Show()
	else
		BWMalygosStacks.time:Hide()
	end
	
	BWMalygosStacks.count:SetText("|cFFFFFFFF"..debuffStacks.."|r")
end

function MalygosStacks:UpdateComboPoints(targetChange)
	if vehicleId then
		local cp = GetComboPoints("vehicle", "target");
		if not targetChange and cp == 0 and comboPoints > 0 then
			spentPoints = comboPoints
		end
		comboPoints = cp
		
		for i = 1, 5 do
			if comboPoints >= i then
				BWMalygosStacks.comboPoints["combo"..i]:Show()
			else
				BWMalygosStacks.comboPoints["combo"..i]:Hide()
			end
		end
		BWMalygosStacks.comboPoints:Show()
	end
end

function MalygosStacks:UNIT_ENTERED_VEHICLE(_, unitId)
	guid = tonumber(string.sub(UnitGUID("vehicle"), -12, -7), 16)
	--print("UNIT_ENTERED_VEHICLE " .. guid)
	if unitId == "player" and (guid == 30161 or guid == 32535) then
		vehicleId = UnitGUID("vehicle")
		comboPoints = 0
		spentPoints = 0
		debuffStacks = 0
		expirationTime = 0
		
		if not BWMalygosStacks then
			self:SetupFrames()
		end
		BWMalygosStacks:Show()
		
		self:ScheduleRepeatingTimer(self.Update, .1, self)
		
		self:RegisterEvent("PLAYER_TARGET_CHANGED", function() self:UpdateComboPoints(true) end)
		self:RegisterEvent("UNIT_COMBO_POINTS", function() self:UpdateComboPoints(false) end)
		self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	end
end

function MalygosStacks:UNIT_EXITED_VEHICLE(_, unitId)
	if vehicleId and unitId == "player" then
		BWMalygosStacks:Hide()
		
		vehicleId = nil
		comboPoints = 0
		spentPoints = 0
		debuffStacks = 0
		expirationTime = 0
		
		self:CancelAllTimers()
		
		self:UnregisterEvent("PLAYER_TARGET_CHANGED")
		self:UnregisterEvent("UNIT_COMBO_POINTS")
		self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	end
end

do
	local debuffName = GetSpellInfo(56092) --61621 used when? i'm lazy.
	function MalygosStacks:COMBAT_LOG_EVENT_UNFILTERED(_, _, event, srcGUID, _, _, _, _, _, ...)
		if vehicleId and srcGUID == vehicleId and select(2, ...) == debuffName then
			if event == "SPELL_AURA_APPLIED" or event == "SPELL_CAST_SUCCESS" then
				
				local cp = comboPoints
				if event == "SPELL_AURA_APPLIED" then
					--cast_success hits before aura_applied and snatches the combo point count
					debuffStacks = 0
					cp = spentPoints
				end
				debuffStacks = debuffStacks + 1
				expirationTime = GetTime() + ((cp - 1) * 4) + 6
				
			elseif event == "SPELL_PERIODIC_DAMAGE" then
				
				local amount, overkill, _, resisted, blocked = select(4, ...)
				local totalDamage = amount + overkill + (resisted or 0) + (blocked or 0)
				local stacks = floor(totalDamage / 1500) --see a lot of 1545, 3090, etc.
				
				if debuffStacks > 0 and abs(stacks - debuffStacks) > 1 then
					--check to make sure we don't get toooo out of sync, prevent jitter
					debuffStacks = stacks
				end
				
			elseif event == "SPELL_AURA_REMOVED" then
				
				debuffStacks = 0
				expirationTime = 0
				
			end
		end
	end
end
